-----------------------------------------------------------------------------------------------
-- Client Lua Script for ViragsSaveKeybind
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ViragsSaveKeybind Module Definition
-----------------------------------------------------------------------------------------------
local ViragsSaveKeybind = {} 
local HELP_COMMAND = "vsk"
local SAVE_COMMAND = "vsksave"
local LOAD_COMMAND = "vskload"
local DELETE_COMMAND = "vskdelete"
local SAVED_COMMAND = "vsksaved"
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ViragsSaveKeybind:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	self.tSavedKeys = {}
    -- initialize variables here

    return o
end

function ViragsSaveKeybind:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

function ViragsSaveKeybind:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ViragsSaveKeybind.xml")
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "VskMain", nil, self)
	if self.wndMain == nil then
		Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
		return
	end
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function ViragsSaveKeybind:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndScroll = self.wndMain:FindChild("Scroll")
		self.wndSaveDialog = self.wndMain:FindChild("SaveDialog")
		self.btnApplyBinds = self.wndMain:FindChild("LoadBtn")
		self.strSelectedName = nil
		Apollo.RegisterSlashCommand(HELP_COMMAND, "OnHelpCommand", self)
		Apollo.RegisterSlashCommand(SAVE_COMMAND, "OnSaveCommand", self)
		Apollo.RegisterSlashCommand(LOAD_COMMAND, "OnLoadCommand", self)
		Apollo.RegisterSlashCommand(DELETE_COMMAND, "OnDeleteCommand", self)
		Apollo.RegisterSlashCommand(SAVED_COMMAND, "OnSavedCommand", self)
	end
end

function ViragsSaveKeybind:OnOpen()
	self.wndMain:Show(true)
	self:ConfigureMainFrame()
end

function ViragsSaveKeybind:ConfigureMainFrame()
	self:RefreshList()
end

function ViragsSaveKeybind:RefreshList()
	self.wndScroll:DestroyChildren()
	for k,v in pairs(self.tSavedKeys) do
		local wndListItem = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndScroll, self)
		self.wndScroll:ArrangeChildrenVert(0)
		wndListItem:FindChild("TextBox"):SetText(k)
		wndListItem:FindChild("ListItemButton"):SetData(k)
	end
end


function ViragsSaveKeybind:OnClose()
	self.wndMain:Show(false)
end


function ViragsSaveKeybind:OnSaveCommand(strSlash, strName)
	if strName == "" then return end
	self.tSavedKeys[strName] = GameLib.GetKeyBindings()
	self:Print(strName .. " saved.")
end
--[[
function ViragsSaveKeybind:OnLoadCommand(strSlash, strName)
	if strName == "" then return end
	if self.tSavedKeys[strName] ~= nil then
		GameLib.SetKeyBindings(self.tSavedKeys[strName])
		self:Print(strName .. " loaded.")
	else 
		self:Print("Invalid name, type /vsksaved to see saved KeyBindings")
	end
end
--]]

function ViragsSaveKeybind:OnDeleteCommand(strSlash, strName)
	if strName == "" then return end
	if self.tSavedKeys[strName] ~= nil then
		self.tSavedKeys[strName] = nil
		self:Print(strName .. " deleted.")
	else 
		self:Print("Invalid name, type /vsksaved to see saved KeyBindings")
	end
end

function ViragsSaveKeybind:OnHelpCommand(strSlash, strName)
	self:OnOpen()
	self:Print("List of commands : \n/" .. HELP_COMMAND .. " \n/" .. SAVE_COMMAND .. " name \n/" .. LOAD_COMMAND
	.. " name \n/" .. DELETE_COMMAND .. " name \n/" .. SAVED_COMMAND)
end

function ViragsSaveKeybind:OnSavedCommand()
	self:Print("Saved KeyBindings: ")
	for k, v in pairs (self.tSavedKeys or {}) do
		self:Print("\n" .. k )
	end
end


function ViragsSaveKeybind:Print(msg)
    if msg == nil then return end
    ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, msg, "Virag's Save Keybind")
end

---------------------------------------------------------------------------------------------------
-- VskMain Functions
---------------------------------------------------------------------------------------------------

function ViragsSaveKeybind:OnSaveBinds( wndHandler, wndControl, eMouseButton )
	self.wndSaveDialog:Show(true)
end

function ViragsSaveKeybind:OnConfirmSave( wndHandler, wndControl, eMouseButton )
	local strSave = self.wndSaveDialog:FindChild("EditBox"):GetText()
	if strSave ~= nil and strSave ~= "" then
		self:OnSaveCommand("", strSave)
		self:RefreshList()
		self.wndSaveDialog:Show(false)
		self.wndSaveDialog:FindChild("EditBox"):SetText("")
	end
end

function ViragsSaveKeybind:OnCancelSave( wndHandler, wndControl, eMouseButton )
	self.wndSaveDialog:Show(false)
end


function ViragsSaveKeybind:OnLoadBinds( wndHandler, wndControl, eMouseButton )
	if self.strSelectedName ~= nil then
		--self:OnLoadCommand("", self.strSelectedName)
		self:RefreshList()
	end
end

function ViragsSaveKeybind:OnDeleteBinds( wndHandler, wndControl, eMouseButton )
	if self.strSelectedName ~= nil then
		self:OnDeleteCommand("", self.strSelectedName)
		self:RefreshList()
	end
end

---------------------------------------------------------------------------------------------------
-- ListItem Functions
---------------------------------------------------------------------------------------------------

function ViragsSaveKeybind:OnListItemCheck( wndHandler, wndControl, eMouseButton )

	self.strSelectedName = wndHandler:GetData()
	local binds = self.tSavedKeys[self.strSelectedName]
	if binds then
		self.btnApplyBinds:SetActionData(GameLib.CodeEnumConfirmButtonType.SaveKeyBinding,  binds )
	end
end

function ViragsSaveKeybind:OnListItemUncheck( wndHandler, wndControl, eMouseButton )
		self.strSelectedName = nil
end

function ViragsSaveKeybind:OnSave(eType)
    if eType ~= GameLib.CodeEnumAddonSaveLevel.General then
        return nil
    end
	local posLeft, posTop = self.wndMain:GetPos()
	return { tKeys = self.tSavedKeys, tMainPos = { left = posLeft, top = posTop } }
end

function ViragsSaveKeybind:OnRestore(eType, tData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.General then
        return nil
    end
	if tData.tKeys ~= nil then
		self.tSavedKeys = tData.tKeys
	end
	if tData.tMainPos ~= nil then
        self.wndMain:Move(tData.tMainPos.left, tData.tMainPos.top, self.wndMain:GetWidth(), self.wndMain:GetHeight())
	end
end

local ViragsSaveKeybindInst = ViragsSaveKeybind:new()
ViragsSaveKeybindInst:Init()



